-- todo: add "something" to indicate whether a redundant index may be required by an Innodb foreign key

CREATE OR REPLACE VIEW
I_S_REDUNDANT_INDEXES
AS
SELECT     l.TABLE_SCHEMA
,          l.TABLE_NAME
,          CASE
               WHEN l.COLUMNS = r.COLUMNS
               AND (l.IS_UNIQUE = r.IS_UNIQUE)
               THEN GREATEST(l.INDEX_NAME, r.INDEX_NAME)
               ELSE l.INDEX_NAME
           END       REDUNDANT_INDEX_NAME
,          GROUP_CONCAT(
               DISTINCT
               CASE
                   WHEN l.COLUMNS = r.COLUMNS
                   AND (l.IS_UNIQUE = r.IS_UNIQUE)
                   THEN LEAST(l.INDEX_NAME, r.INDEX_NAME)
                   ELSE r.INDEX_NAME
               END
           )         INDEX_NAME
FROM       I_S_INDEXES l
INNER JOIN I_S_INDEXES r
ON         l.TABLE_SCHEMA = r.TABLE_SCHEMA          -- index on the same table
AND        l.TABLE_NAME   = r.TABLE_NAME
AND        l.INDEX_NAME  != r.INDEX_NAME            -- but not identical
AND        l.INDEX_TYPE   = r.INDEX_TYPE
AND        CASE
               WHEN l.COLUMNS     = r.COLUMNS       -- we require column equality
               AND (l.IS_UNIQUE  = 'NO'             -- redundant if not unique
                 OR l.IS_UNIQUE   = r.IS_UNIQUE)    -- or if same uniqueness as other
               THEN TRUE
               WHEN l.INDEX_TYPE  = 'BTREE'         -- when BTREE
               AND  INSTR(r.COLUMNS, l.COLUMNS) = 1 -- and l is a prefix
               AND  l.IS_UNIQUE   = 'NO'            -- redundant if not unique
               THEN TRUE
               ELSE FALSE
           END
GROUP BY   l.TABLE_SCHEMA
,          l.TABLE_NAME
,          REDUNDANT_INDEX_NAME